// PurchaseOrderBean.java

package purchaseOrderService.ejb;

import java.rmi.*;
import java.sql.*;
import javax.ejb.*;
import javax.naming.*;
import javax.sql.*;
import java.util.Collection;

//////////////////////////////////////////////////////////////////////  
public abstract class PurchaseOrderBean implements EntityBean
{
    EntityContext ctx;

    final   String  DSName           = "java:comp/env/jdbc/nonEmulatedDS";
    final   String  productBeanName  = "java:comp/env/ejb/Product";
    final   String  lineItemBeanName = "java:comp/env/ejb/LineItem";

    // Get methods
    public  abstract  Integer    getPONumber()          throws RemoteException;
    public  abstract  String     getCustomerName()      throws RemoteException;
    public  abstract  String     getCreditCardNumber()  throws RemoteException;
    public  abstract  String     getStatus()            throws RemoteException;
    public  abstract  Date       getDeliveryDate()      throws RemoteException;
    public  abstract  Float      getTotalCost()         throws RemoteException;
    public  abstract  Collection getLineItems()         throws RemoteException;

    // Set methods
    public  abstract  void  setPONumber(Integer poNumber)           
      throws RemoteException;
    public  abstract  void  setCustomerName(String customerName)    
      throws RemoteException;
    public  abstract  void  setCreditCardNumber(String cardNumber)  
      throws RemoteException;
    public  abstract  void  setStatus(String status)                
      throws RemoteException;
    public  abstract  void  setDeliveryDate(Date deliveryDate)      
      throws RemoteException;
    public  abstract  void  setTotalCost(Float tc)                  
      throws RemoteException;
    public  abstract  void  setLineItems(Collection lineItems)      
      throws RemoteException;

    //**************** Entity Bean specific APIs *******************//
    //////////////////////////////////////////////////////////////////////  
    public Integer ejbCreate(String customerName, String creditCardNumber, 
                             String[] products, int[] quantities)
      throws RemoteException, CreateException
    {
      System.out.println("=======>PurchaseOrderBean.ejbCreate(): begin");

      // 1. confirm that we recd properly...
      if(customerName == null || creditCardNumber == null ||
         products == null || quantities == null)
        throw new CreateException(
          "cannot create PO with unknown parameters...");
      if(products.length != quantities.length)
        throw new CreateException(
          "productNames.length MUST equal quantities.length");
    	if(products.length < 0)
        throw new CreateException("productNames.length MUST be atleast 1");

    	try
      {
        // 2. create Purchase order number
        int poNum = createPONumber();
        if(poNum <= 0) 
          throw new CreateException("could not create a po");

  	    // 3. set cmp-fields
        setPONumber(new Integer(poNum));
        setCustomerName(customerName.toUpperCase());
        setCreditCardNumber(creditCardNumber);
        setStatus(Status.getString(Status.STATUS_NEWPO));

       // 4. evaluate the total cost
       float cost = evaluateTotalCost(products, quantities);
        if(cost != 0.0)
        	setTotalCost(new Float(cost));
      }
      catch(TestException e)
    	{
        e.printStackTrace();
        throw new CreateException("cannot create PO: " + e.getMessage());
    	}
      catch(FinderException e)
      {
        System.out.println(
          "         : FinderException caught in ejbCreate of POBean....");
  	    e.printStackTrace();
        throw new CreateException("cannot create PO: " + e.getMessage());
      }
      catch(Throwable e)
      {
        e.printStackTrace();
        throw new CreateException("cannot create PO: " + e.getMessage());
      }
      finally
      {
        System.out.println(
          "=======>PurchaseOrderBean.ejbCreate(): end---poNumber= " +
          getPONumber());
        return getPONumber();
      }
    }

    //////////////////////////////////////////////////////////////////////  
    public void ejbPostCreate(String customerName, String creditCardNumber,
                              String[] products, int[] quantities)
      throws RemoteException, CreateException
    {
      System.out.println("=======>PurchaseOrderBean.ejbPostCreate(): begin");

    	int i = 0;
      try
      {
        // 1. get a JNDI context
        Context context = new InitialContext();
	    
        // 2. lookup the lineItemHome
        LineItemHome lineItemHome =
          (LineItemHome)context.lookup(lineItemBeanName);

        // 3. for every productName create a lineItem
        for(; i < products.length; i++)
          getLineItems().add(lineItemHome.create(products[i], quantities[i]));
      }
      catch(NamingException e)
      {
        System.out.println(
          "         : NamingException: for " + lineItemBeanName);
        e.printStackTrace();
        throw new RemoteException(
          "PurchaseOrderBean.ejbPostCreate()---NamingException: for " +
          lineItemBeanName);
      }
      catch(CreateException e)
      {
        System.out.println("         : CreateException: for " + products[i]);
        e.printStackTrace();
        throw new CreateException(
          "Could not create a lineItem with: " + products[i]);
      }
      finally
      {
        System.out.println(
        "=======>PurchaseOrderBean.ejbPostCreate(): end---getLineItems().size= "
        + getLineItems().size());
      }
    }

    public void ejbPostCreate(Integer poNumber)
      throws RemoteException, CreateException
    {  // called just after bean created
    }

    public void ejbStore()
      throws RemoteException
    {  // called when bean persisted
    }

    public void ejbLoad()
    {   // called when bean loaded
    }

    public void ejbRemove()
    { // called when bean removed
    }

    public void ejbActivate()
    { // called when bean activated
    }

    public void ejbPassivate()
    { // called when bean deactivated
    }

    public void setEntityContext(EntityContext ctx)
    {
      this.ctx = ctx;
    }

    public void unsetEntityContext()
    {
      this.ctx = null;
    }

    public EJBHome getEJBHome()
    {
      return ctx.getEJBHome();
    }

    public Handle getHandle() throws RemoteException
    {
      return ctx.getEJBObject().getHandle();
    }

    public Object getPrimaryKey() throws RemoteException
    {
      return ctx.getEJBObject().getPrimaryKey();
    }

    public boolean isIdentical(EJBObject remote) throws RemoteException
    {
      return ctx.getEJBObject().isIdentical(remote);
    }

    public void remove() throws RemoveException, RemoteException
    {
    }

    /////////////////////////////  PRIVATE APIs //////////////////////////
    //////////////////////////////////////////////////////////////////////  
    private float evaluateTotalCost(String[] products, int[] quantities)
      throws FinderException, TestException
    {
      System.out.println(
        "=======>PurchaseOrderBean.evaluateTotalCost(): begin");

      ProductHome productHome = null;
      float totCost = 0/1;
      int i=0;
      try
      {
        // 1. get a JNDI context
        Context context = new InitialContext();

  	    // 2. lookup the productHome
        productHome = (ProductHome) context.lookup(productBeanName);

        for(; i < products.length; i++)
        {
          // 3. find the remote-inteface for the productHome
          ProductRemote productRemote =
            productHome.findByPrimaryKey(products[i].toUpperCase());

      		// 4. get the unit price
          Float unitPrice = productRemote.getUnitPrice();

      		// 5. compute the total-cost
          totCost += unitPrice.floatValue() * quantities[i];
        }
      }
      catch(NamingException e)
      {
        System.out.println(
            "         : NamingException in POBean--evaluateTotalCost: " +
            productBeanName);
    	}
      catch(FinderException e)
      {
        System.out.println(
        "         : FinderException in POBean--evaluateTotalCost for product: "
        + products[i].toUpperCase());
        throw e;
      }
      catch(Throwable e)
      {
  	    System.out.println(
        "         : Throwable exception caught in evaluateTotalCost of product: "
        + products[i].toUpperCase());
        e.printStackTrace();
        throw new TestException("cannot create PO: " + e.getMessage());
    	}
      finally
      {
        System.out.println(
        "=======>PurchaseOrderBean.evaluateTotalCost(): end---Total Cost = " +
        totCost);
        return totCost;
      }
    }

    //////////////////////////////////////////////////////////////////////  
    private int createPONumber() throws TestException
    {
    	System.out.println("=======>PurchaseOrderBean.createPONumber(): begin");
      PreparedStatement ps = null;
      ResultSet rs = null;
      Connection conn = null;

    	try
      {
        // 0. create a JNDI Context
        Context  context = new  InitialContext();
	    
        // 1. lookup the client's Datasource
        DataSource  userds = (DataSource) context.lookup(DSName);
        if (userds == null)
        throw new TestException("No DataSource found in JNDI with Name = " +
                                DSName);
  
	    // 2. get a connection
	    conn = userds.getConnection();
  
	    // create and execute a ps..
	    ps = conn.prepareStatement("select po_seq.nextval from dual");
	    rs = ps.executeQuery();
	    if (!rs.next())
    		throw new TestException("select po_seq.nextval from dual FAILED");
	    
	    java.math.BigDecimal bd = rs.getBigDecimal(1);
	    int poNum = bd.intValue();

	    System.out.println("         : poNum=" + poNum);
	    return poNum;
      }
      catch(NamingException e)
      {
        System.out.println("         : NamingException: for " + DSName);
        e.printStackTrace();
        throw new TestException(
          "PurchaseOrderBean: createPONumber:--NamingException: for " + DSName);
      }
      catch(SQLException e)
      {
        System.out.println("         : SQLException: for " + DSName);
        throw new TestException(
          "PurchaseOrderBean: createPONumber:--SQLException....");
      }
      catch(Throwable e)
      {
        System.out.println("         : Throwable: exception");
        e.printStackTrace();
        throw new TestException(
          "PurchaseOrderBean: createPONumber:--Throwable....");
    	}
      finally
      {
        try
        {
          if(rs != null)
            rs.close();
          if(ps != null)
            ps.close();
          if(conn != null && !conn.isClosed())
            conn.close();
        }
        catch(SQLException e)
	    {
        System.out.println("         : SQLException in finally");
	    }

	    System.out.println("=======>PurchaseOrderBean.createPONumber(): end");
    }
  }
}
