// PurchaseorderServiceBean.java

package purchaseOrderService.ejb;

import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;

import oracle.aurora.jem.*;

//////////////////////////////////////////////////////////////////////  
public class PurchaseOrderServiceBean implements SessionBean
{
    SessionContext ctx;
    final static String purchaseOrderEntityBean =
      "java:comp/env/ejb/PurchaseOrder";

    // JEM-Bean Names
    final static String jemPurchaseOrderServiceBeanName =
      "JEMPurchaseOrderService";
    final static String jemCreditServiceName =
      "JEMCreditServiceBean";
    final static String jemInventoryServiceName =
      "JEMInventoryServiceBean";

    //**************** Session Bean specific APIs *******************//
    public void setSessionContext(SessionContext ctxArg)
    {
      ctx = ctxArg;
    }
    public void ejbCreate() throws RemoteException, CreateException 
    {
      System.out.println(
        "=======>PurchaseorderServiceBean.ejbCreate(): begin/end");
    }
    public void ejbActivate() {}
    public void ejbPassivate() {}
    public void ejbRemove() {}

    ///////////////////////////  PUBLIC API //////////////////////////////////
    public String getStatus(int poNumber)
      throws RemoteException, TestException
    {
      System.out.println(
        "=======>PurchaseOrderServiceBean.getStatus(): begin---poNumber= " +
        poNumber);
      String status = "STATUS_INVALID";

      try
      {
        // 1. lookup purhanseOrder Bean
        PurchaseOrderRemote poRemote = lookupPOBean(new Integer(poNumber));

        // 2. retrieve status
        status = poRemote.getStatus();
      }
      finally
      {
        System.out.println(
          "=======>PurchaseOrderServiceBean.getStatus(): end---poNumber= " +
          poNumber + "  status=" + status);
          return status;
      }
    }

    /////////////////////// PUBLIC JEM API  /////////////////////////////////
    public int takeOrder(String clientName, String creditCardNumber,
                         String[] productNames, int[] quantities)
      throws RemoteException, TestException
    {
      System.out.println("=======>PurchaseOrderServiceBean.takeOrder(): begin");
      System.out.println("         : clientName = " + clientName);
      System.out.println("         : creditCardNumber = " + creditCardNumber);

      if(productNames == null || quantities == null ||
         productNames.length != quantities.length)
        throw new TestException(
          "productNames.length MUST equal quantities.length");

      if(productNames.length < 0)
        throw new TestException("productNames.length MUST be atleast 1");

      for(int i =0; productNames.length > 0 && i < productNames.length; i++)
        System.out.println("         : (productName, quantity) = " +
                           productNames[i] + ", " + quantities[i]);

      int poNumber = 0;
      try
      {
        // 1. Create Purchase Order Entity Bean, and get PO Number in return
        poNumber = createPO(clientName, creditCardNumber,
                            productNames, quantities);
        System.out.println("         : poNumber= " + poNumber);

        // 2. Get the current AC4J Reaction
        JEMReaction currentAC4JReaction = (JEMReaction)JEMReaction.getReaction();

        // 3. Make an asynchronous call to the processOrder reaction on this
        //    Bean, so can return PO Number to client now
        JEMEmitToken purchaseReq = startProcessingPO(currentAC4JReaction,
                                                     poNumber,
                                                     productNames,
                                                     quantities);
      if(purchaseReq == null)
        throw new TestException("Sending processOrder request failed");
      }
      catch(JEMException  e)
      {
        throw new TestException(e.getMessage());
      }
      finally
      {
        System.out.println("=======>PurchaseOrderServiceBean.takeOrder(): end");

          // 4. Return the PO Number to the client
          return poNumber;
      }
    }

    /////////////////////// PUBLIC JEM API  //////////////////////////////////
    public void processOrder(int poNumber, String[] productNames,
                             int[] quantities)
      throws RemoteException, TestException
    {
      System.out.println(
        "=======>PurchaseOrderServiceBean.processOrder(): begin---"
        + "poNumber= " + poNumber);

      PurchaseOrderRemote poRemote = null;
      try
      {
        // 1. Get the current AC4J Reaction
        JEMReaction  currentAC4JReaction =
          (JEMReaction)JEMReaction.getReaction();

        // 2. Look up purchase order Entiry Bean, so can pass reference to
        //    Credit and Inventory Services
        poRemote = lookupPOBean(new Integer(poNumber));
        if(poRemote == null)
          throw new TestException("Could not find po with poNumber= " +
                                  poNumber);

        // 3. Make an asynchronous call to the Credit Service
        JEMEmitToken creditToken = callCreditService(currentAC4JReaction,
                                                     poRemote);
        if(creditToken == null)
          throw new TestException("Sending request to Credit Service failed");

        // 4. Make an asynchronous call to the Inventory Service
        JEMEmitToken inventoryToken = callInventoryService(currentAC4JReaction, 
                                                           productNames,
                                                           quantities);
        if(inventoryToken == null)
          throw new TestException("Sending request to Inventory Service failed");

        // 5. Register processOrderCallback Reaction
        registerCallback(currentAC4JReaction,
                         new JEMEmitToken[] {inventoryToken, creditToken});

        // 6. Update Purchase Order status to INPROCESS
        poRemote.setStatus(Status.getString(Status.STATUS_INPROCESS));
      }
      catch(JEMException  JEMexc)
      {
        throw new TestException(JEMexc.getMessage());
      }
      finally
      {
        System.out.print(
          "=======>PurchaseOrderServiceBean.processOrder(): end---");
        if(poRemote != null)
          System.out.println("status= " + poRemote.getStatus());
        else
          System.out.println();
      } 
    }

    /////////////////////// PUBLIC JEM API  /////////////////////////////////
    public void processOrderCallback(boolean[] invInfo, String credInfo)
      throws RemoteException, TestException
    {
      System.out.println(
        "=======>PurchaseOrderServiceBean.processOrderCallback(): begin." +
        "credInfo=" + credInfo);

      PurchaseOrderRemote poRemote = null;
      Integer poNumber = null;

      try
      {
        //    This Reaction (processOrderCallback) was invoked asynchronously by
        //    it's parent Reaction (processOrder) by doing 
        //    "JEMReaction.registerReaction".
        //
        //    This reaction can access all it's parent's variables by using 
        //    the JEMProcess concept as shown in the following 3 steps.
        //
        //    In this case, it is the PO Number that is accessed. 

        // 1. Obtain the current AC4J Process handle
        JEMProcess  currentAC4JProcess = (JEMProcess) JEMProcess.getProcess();

        // 2. Retrieve the input parameters of parent reaction: processOrder
        JEMTuple inTuple = currentAC4JProcess.getInTupleByIndx(0);

        // 3. Get PO Number from parent's input parameters
        poNumber = (Integer) inTuple.getObjInst();
        System.out.println("=======>PurchaseOrderServiceBean.callBackMethod()" +
                           ": poNumber= " + poNumber);

        // 4. Look up purhanseOrder Bean
        poRemote = lookupPOBean(poNumber);
        if(poRemote == null)
          throw new TestException("Could not find po with poNumber= " +
                                  poNumber);

        // 5. Start updating the PO status depending on the replies...
        poRemote.setStatus(Status.getString(Status.STATUS_PROCESSED));

        // 6. Check the credit info
        int local_status = Status.STATUS_PROCESSED;
        if(credInfo != null)
        {
          if(credInfo.equalsIgnoreCase("credit approved"))
            local_status = Status.STATUS_VALID_CREDIT;
         else if(credInfo.equalsIgnoreCase("credit failed"))
            local_status = Status.STATUS_CANCELLED_NOCREDIT;
          else if(credInfo.equalsIgnoreCase("Invalid Credit Card"))
            local_status = Status.STATUS_INVALID_CREDIT_CARD;
        }
        else
          local_status = Status.STATUS_CANCELLED_NOCREDIT;

        // 7. Check the inventory info
        for(int i=0;
          local_status == Status.STATUS_VALID_CREDIT && i<invInfo.length;
          i++)
        {
          System.out.println("         : invInfo["+ i + "] = " + invInfo[i]); 
          if(!invInfo[i])
            local_status = Status.STATUS_CANCELLED_NOINV;
        } 

        // 8. reset the final status of the Purchase Order
        if(local_status == Status.STATUS_VALID_CREDIT)
          poRemote.setStatus(Status.getString(Status.STATUS_SHIPPED));
        else
          poRemote.setStatus(Status.getString(local_status));
      }
      catch(JEMException  JEMexc)
      {
        throw new TestException(JEMexc.getMessage());
      }
      finally
      {
        System.out.print(
        "=======>PurchaseOrderServiceBean.callBackMethod(): end---credInfo=" + 
          credInfo + "  poNumber=" + poNumber);
        if(poRemote != null)
          System.out.println("  currentStatus= " + poRemote.getStatus());
        else
          System.out.println();
      }
    } 

    // ************************  PRIVATE APIs ************************* //
    //////////////////////////////////////////////////////////////////////  
    private PurchaseOrderRemote lookupPOBean(Integer poNumber)
      throws RemoteException, TestException
    {
      PurchaseOrderRemote poRemote = null;

      try
      {
        // 1. create a JNDI Context
        Context context = new InitialContext();
      
        // 2. lookup the Purchaseorder-Bean
       PurchaseOrderHome poHome =
        (PurchaseOrderHome)context.lookup(purchaseOrderEntityBean);

        // 3. findByPK
        poRemote = poHome.findByPrimaryKey(poNumber);
      }
      catch(NamingException  ne)
      {
        throw new TestException(
            "NamingException: Could not find po with poNumber=" + poNumber);
      }
      catch(FinderException  ne)
      {
        throw new TestException(
            "FinderException: Could not Find po with poNumber=" + poNumber);
      }

      return poRemote;
    }

    //////////////////////////////////////////////////////////////////////  
    private int createPO(String clientName, String creditCardNumber,
                         String[] productNames, int[] quantities)
      throws TestException
    {
      System.out.println("=======>PurchaseOrderServiceBean.createPO(): begin");
      int poNumber = 0;

      try
      {
        // 1. Create a JNDI Context
        Context context = new InitialContext();

        // 2. Look up the Purchaseorder-Bean Home-interface
        PurchaseOrderHome poHome = (PurchaseOrderHome)
                                   context.lookup(purchaseOrderEntityBean);

        // 3. Create a NEW-PurchaseOrder Entity
        PurchaseOrderRemote poRemote = poHome.create(clientName,
                                                     creditCardNumber,
                                                     productNames,
                                                     quantities);

        // 4. Get PO Number
        poNumber = poRemote.getPONumber().intValue();
        System.out.println("       createPO()---poNumber= " + poNumber);
       }
      catch(Exception  e)
      {
        throw new TestException(
          "Exception: " + e.getMessage());
      }
      finally
      {
        System.out.println("=======>PurchaseOrderServiceBean.createPO(): end");
        return poNumber;
      }
    }

    //////////////////////////////////////////////////////////////////////  
    private JEMEmitToken startProcessingPO(JEMReaction currentAC4JReaction, 
                                           int poNumber, 
                                           String[] productNames,
                                           int[] quantities)
      throws TestException
    {
      System.out.println(
          "=======>PurchaseOrderServiceBean.startProcessingPOrder(): begin");

      JEMEmitToken emitToken = null;

      try
      {
        // 1. Create a context
        Context context = new InitialContext();

        // 2. Look up the AC4J handle for this Bean (PurchaseOrderService)
        JEMHandle ac4jPOSBeanHandle =
          (JEMHandle)context.lookup(jemPurchaseOrderServiceBeanName);

        // 3. Prepare input parameter Types for processOrder method
        Class[] inputClassTypes = new Class[] { Integer.TYPE,
                                                productNames.getClass(),
                                                quantities.getClass() };

        // 4. Prepare input parameter Values for processOrder method
        Object[] inputParams = new Object[] { (Object) (new Integer(poNumber)),
                                              (Object) productNames,
                                              (Object) quantities };

        // 5. Make asynchronous call to processOrder
        emitToken =  currentAC4JReaction.call(null, null, ac4jPOSBeanHandle,
                                              "processOrder", inputClassTypes,
                                              null, inputParams, null, null,
                                              null, 0,  0);
      }
      catch(Exception e)
      {
        throw new TestException(
          "Exception: " + e.getMessage());
      }
      finally
      {
        System.out.println(
            "=======>PurchaseOrderServiceBean.startProcessingPOrder(): end");
        return emitToken;
      }
    }

    //////////////////////////////////////////////////////////////////////  
    private JEMEmitToken callCreditService(JEMReaction currentAC4JReaction,
                                           PurchaseOrderRemote poRemote)
      throws RemoteException, JEMException, TestException
    {
      System.out.println(
          "=======>PurchaseOrderServiceBean.callCreditService(): begin");

      // 1. Look up the AC4J Handle for the Credit Service
     JEMHandle creditCardJEMHandle = lookupJEMHandle(jemCreditServiceName);

      // 2. Prepare input parameter Types for the checkCRED method
     Class[] inputClassTypes = new Class[] { String.class,
                                            String.class,
                                            Float.TYPE };

      // 3. Prepare input parameter Values for the checkCRED method
      Object[]  inputParams = new Object[] {
                                      (Object) poRemote.getCustomerName(),
                                      (Object) poRemote.getCreditCardNumber(),
                                      (Object) poRemote.getTotalCost() };

      // 4. Prepare return parameter Type of the checkCRED method
      Class[]   retClassTypes = new  Class[] { String.class };

      // 5. Make an asynchronous call to the checkCRED method
     System.out.println(
        "=======>PurchaseOrderServiceBean.callCreditService(): end");
     return currentAC4JReaction.call(null, null, creditCardJEMHandle,
                                     "checkCRED", inputClassTypes, null,
                                     inputParams, retClassTypes, 
                                     null, null, 0,  0);
    }

    //////////////////////////////////////////////////////////////////////  
    private JEMEmitToken callInventoryService(JEMReaction currentAC4JReaction, 
                                       String[] productNames, int[] quantities)
      throws RemoteException, JEMException, TestException
    {
      System.out.println(
         "=======>PurchaseOrderServiceBean.callInventoryService(): begin");

      boolean[] dummyVariable = new boolean[1];

      // 1. Look up the AC4J Handle for the Inventory Service Active EJB
      JEMHandle inventoryJEMHandle = lookupJEMHandle(jemInventoryServiceName);

      // 2. Prepare input parameter Types for the checkINV method
      Class[] inputClassTypes = new Class[] { productNames.getClass(),
                                              quantities.getClass() };

      // 3. Prepare input parameter Values for the checkINV method
      Object[]  inputParams = new Object[] { (Object) productNames,
                                             (Object) quantities };

      // 4. Prepare return parameter Type of the checkINV method
      Class[]   retClassTypes = new  Class[] { dummyVariable.getClass() };

      // 5. Make an asynchronous call on the checkINV method
      System.out.println(
          "=======>PurchaseOrderServiceBean.callInventoryService(): end");
      return currentAC4JReaction.call(null, null, inventoryJEMHandle,
                                      "checkINV", inputClassTypes, null,
                                      inputParams, retClassTypes,
                                      null, null, 0,  0);
    }

    //////////////////////////////////////////////////////////////////////  
    private void registerCallback(JEMReaction currentAC4JReaction,
                                  JEMEmitToken[] requests)
      throws JEMException
    {
      System.out.println(
        "=======>PurchaseOrderServiceBean.registerAsyncRespHandler(): begin");

      boolean[] dummyVariable = new boolean[1];

      // 1. Prepare input parameter Types for the processOrderCallback method
      Class[] inputClassTypes = new Class[] { dummyVariable.getClass(),
                                              String.class };

      // 2. Using the request tokens and input parameter Types register a
      //    reaction with method processOrderCallback
      currentAC4JReaction.registerReaction("processOrderCallback",
                                           inputClassTypes, requests, 1,
                                           null, null, 0);

      System.out.println(
          "=======>PurchaseOrderServiceBean.registerAsyncRespHandler(): end");
    }

    //////////////////////////////////////////////////////////////////////
    private JEMHandle lookupJEMHandle(String jemName) throws TestException
    {
      JEMHandle  jemHandle = null;
      try
      {

        // create a naming context
        Context context = new InitialContext();

        // look up the jem-handle
        jemHandle = (JEMHandle)context.lookup(jemName);
        if(jemHandle == null)
            throw new TestException("JEM Bean not found");
      }
      catch(NamingException namexc)
      {
        throw new TestException("JEM Bean not found");
      } 

      return jemHandle;
    }
}
