#!/bin/sh
#
# postinstall.sh
#
# Copyright (c) 2001, 2002, Oracle Corporation.  All rights reserved.  
#
#    NAME
#      postinstall.sh - <one-line expansion of the name>
#
#    DESCRIPTION
#      This script loads the JAZN demo data into LDAP(OID) by default. 
#      It can be used to load the jazn schema data as well as demo.
#
#
#    NOTES
#
#        Please refers to README for detail instructions.
#        This file should be instantiated with the ORACLE_HOME variable subsituted properly.  
#        ** Make sure our ORACLE_HOME environment variable is set properly. **       
#
############################################################################################


while test $# -gt 0
do
  case $1 in
    -h) shift; LDAP_HOST="$1";;
    -p) shift; LDAP_PORT="$1";;
    -D) shift; BIND_DN="$1";;
    -w) shift; BIND_PASSWD="$1";;
    -load) shift; LOAD="$1";;
    -help) echo "Usage: postinstall.sh [-h ldaphost] [-p ldapport] [-D binddn] [-w password] [-load <Demo|All>]"; exit 1;;
   esac
  shift
done



# set Oracle Home
set ORACLE_HOME=%ORACLE_HOME%
export ORACLE_HOME

# Get LDAP host
if [ -z "${LDAP_HOST}" ]; then 
    echo "Error: LDAP HOST not set!" 
    echo "Usage: postinstall.sh [-h ldaphost] [-p ldapport] [-D binddn] [-w password] [-load <Demo|All>]"; 
    exit 1 
fi

export LDAP_HOST

# Get LDAP host port number
if [ -z "${LDAP_PORT}" ]; then
    LDAP_PORT=389
fi
export LDAP_PORT

  
# Get LDAP Bind DN

if [ -z "${BIND_DN}" ]; then
    BIND_DN=cn=orcladmin
fi
export BIND_DN

# Get LDAP Bind Password
if [ -z "${BIND_PASSWD}" ]; then 
    echo "Error: BIND PASSWORD not set!" 
    exit 1 
fi
#export BIND_PASSWD

#Load Demo by default
if [ "${LOAD}" = "" ]; then 
    LOAD=Demo
fi 

#set install directory where the ldif files are expected
INSTALLDIR=%ORACLE_HOME%/j2ee/home/jazn/install
export INSTALLDIR
#echo $INSTALLDIR


# check if loading Login Module only
if [ "${LOAD}" = "Demo" ]; then 
    %ORACLE_HOME%/bin/ldapmodify -c -a -p $LDAP_PORT -h $LDAP_HOST -D $BIND_DN -w $BIND_PASSWD -f $INSTALLDIR/oidDemoDataCreateJAZN.ldif >> $INSTALLDIR/postinstall.log
   echo "***Done loading demo data! Please check"
   echo $INSTALLDIR/postinstall.log "for results***"
    exit 0 
fi

# check if loading Login Module only
if [ "${LOAD}" = "LoginModule" ]; then 
         %ORACLE_HOME%/bin/ldapmodify -c -a -p $LDAP_PORT -h $LDAP_HOST -D $BIND_DN -w $BIND_PASSWD -f $INSTALLDIR/oidSchemaCreateJAZNLM.ldif >>$INSTALLDIR/postinstall.log
  echo "***Done loading login module! Please check"
  echo $INSTALLDIR/postinstall.log "for results***"
   
 exit 0 
fi

# check if loading all
if [ "${LOAD}" = "All" ]; then 



  %ORACLE_HOME%/bin/ldapmodify -c -a -p $LDAP_PORT -h $LDAP_HOST -D $BIND_DN -w $BIND_PASSWD -f $INSTALLDIR/oidSchemaCreateJAZN.ldif >> $INSTALLDIR/postinstall.log

  %ORACLE_HOME%/bin/ldapmodify -c -a -p $LDAP_PORT -h $LDAP_HOST -D $BIND_DN -w $BIND_PASSWD -f $INSTALLDIR/oidContextCreateJAZN.ldif >>postinstall.log

  %ORACLE_HOME%/bin/ldapmodify -c -a -p $LDAP_PORT -h $LDAP_HOST -D $BIND_DN -w $BIND_PASSWD -f $INSTALLDIR/oidDemoDataCreateJAZN.ldif >> $INSTALLDIR/postinstall.log

  echo "***Done loading JAZN base schema and demo data! Please check "; 
  echo $INSTALLDIR/postinstall.log "for results***"
  exit 0
fi


echo "Usage: postinstall.sh [-h ldaphost] [-p ldapport] [-D binddn] [-w password] [-load <Demo|All>]";




